package com.fireworksdk.bridge.flutter.handler

import android.content.Context
import com.fireworksdk.bridge.flutter.models.FWSdkModuleCallMethod
import com.fireworksdk.bridge.models.*
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import java.nio.charset.StandardCharsets
import java.security.MessageDigest
import kotlin.collections.HashMap
import android.provider.Settings
import android.annotation.SuppressLint
import android.content.pm.PackageManager
import android.util.Base64
import com.firework.analyticsevents.cta.CtaButtonClickAnalyticsEvent
import com.firework.analyticsevents.player.PlayerLifecycleAnalyticsEvent
import com.firework.analyticsevents.share.ShareButtonAnalyticsEvent
import com.firework.bus.FwAnalyticCallable
import com.firework.sdk.FireworkSdk
import com.firework.sdk.FireworkSdkConfig
import com.fireworksdk.bridge.FWInitializationProvider
import com.fireworksdk.bridge.flutter.utils.FWEventUtils
import com.fireworksdk.bridge.models.enums.FWPlayerLaunchBehavior
import com.fireworksdk.bridge.models.enums.FWVideoPlaybackSubEventName
import com.fireworksdk.bridge.utils.FWConfigUtil
import com.fireworksdk.bridge.utils.FWGlobalDataUtil
import com.fireworksdk.bridge.utils.FWImageLoaderUtil
import com.fireworksdk.bridge.utils.FWLanguageUtil
import com.fireworksdk.bridge.utils.FWLiveStreamUtil
import com.fireworksdk.bridge.utils.FWLogUtils
import org.json.JSONObject


class FWSdkModuleHandler(
  private val context: Context,
  channel: MethodChannel?
) : MethodCallHandler {

  private val methodChannel by weakProperty<MethodChannel?>(channel)

  override fun onMethodCall(call: MethodCall, result: Result) {
    when (call.method) {
      FWSdkModuleCallMethod.Init.rawValue -> {
        val configMap = call.arguments
        val jsonObject = JSONObject(configMap as HashMap<*, *>)
        val sdkInitOptionsModel = FWSDKInitOptionsModelDeserializer.deserialize(jsonObject)
        initSdk(sdkInitOptionsModel)
        FireworkSdk.analytics.register(this)
        result.success(null)
      }

      FWSdkModuleCallMethod.OpenVideoPlayer.rawValue -> {
        val map = call.arguments as HashMap<*, *>
        if (map.size > 0) {
          val url = map["url"] as String
          val configMap = map["config"]
          val jsonObject = JSONObject(configMap as HashMap<*, *>)
          val videoPlayerConfigModel = FWVideoPlayerConfigModelDeserializer.deserialize(jsonObject)

          val videoFeedPropsModel = FWVideoFeedPropsModel(
            videoPlayerConfiguration = videoPlayerConfigModel
          )
          val viewOptionsBuilder = FWConfigUtil.generateViewOptionsBuilder(context, videoFeedPropsModel)
          FireworkSdk.startPlayer(viewOptionsBuilder.build(), url)
        }
        result.success(null)
      }

      FWSdkModuleCallMethod.SetShareBaseURL.rawValue -> {
        val url: String? = call.arguments as String?
        url?.let {
          FWGlobalDataUtil.shareBaseUrl = it
        }
        result.success(null)
      }

      FWSdkModuleCallMethod.SetAdBadgeConfiguration.rawValue -> {
        val configMap = call.arguments
        val jsonObject = JSONObject(configMap as HashMap<*, *>)
        val adBadgeConfigModel = FWAdBadgeConfigModelDeserializer.deserialize(jsonObject)
        FWGlobalDataUtil.adBadgeConfigModel = adBadgeConfigModel ?: FWAdBadgeConfigModel()
        result.success(null)
      }

      FWSdkModuleCallMethod.SetCustomCTAClickEnabled.rawValue -> {
        val value: Boolean? = call.arguments as Boolean?
        FWGlobalDataUtil.customCTAClickEnabled = value?:false
        result.success(null)
      }

      FWSdkModuleCallMethod.SetVideoPlaybackEventEnabled.rawValue -> {
        val value: Boolean? = call.arguments as Boolean?
        FWGlobalDataUtil.videoPlaybackEventEnabled = value?:false
        result.success(null)
      }

      FWSdkModuleCallMethod.ChangeAppLanguage.rawValue -> {
        val activity = FWInitializationProvider.INSTANCE.resumedActivity
        if (activity == null) {
          result.success(false)
          return
        }
        val language: String? = call.arguments as String?
        val changeLanguageResult = FWLanguageUtil.getInstance(activity).changeLanguage(language, activity)
        result.success(changeLanguageResult)
      }

      else -> {
        result.notImplemented()
      }
    }
  }

  private fun initSdk(sdkInitOptions: FWSDKInitOptionsModel?) {
    val clientId = getAppid()
    if (clientId.isNullOrBlank()) {
      val errorMsg = "The appid is invalid"
      FWLogUtils.d { errorMsg }
      FWEventUtils.sendInitFailedEvent(methodChannel, errorMsg)
      return
    }
    val userId = sdkInitOptions?.userId

    val uid = if (userId.isNullOrBlank()) { getHashedAndroidId() } else userId
    val shareBaseURL = sdkInitOptions?.shareBaseURL
    val videoLaunchBehavior = sdkInitOptions?.videoLaunchBehavior

    val configBuilder = FireworkSdkConfig.Builder(context)
      .checksumRequired(false)
      .clientId(clientId)
      .userId(uid)
      .enableCache(true)
    shareBaseURL?.let {
      FWGlobalDataUtil.shareBaseUrl = it
    }

    val livestreamPlayerInitializerList = FWLiveStreamUtil.livestreamPlayerInitializers()
    for (livestreamPlayerInitializer in livestreamPlayerInitializerList) {
      configBuilder.addLivestreamPlayerInitializer(livestreamPlayerInitializer)
    }
    when (videoLaunchBehavior) {
      FWPlayerLaunchBehavior.Default -> {}
      FWPlayerLaunchBehavior.MuteOnFirstLaunch -> {
        configBuilder.muteOnLaunch(true)
      }
      else -> {}
    }

    val imageLoader = FWImageLoaderUtil.getImageLoader()
    if (imageLoader != null) {
      configBuilder.imageLoader(imageLoader)
    }

    val config = configBuilder.build()

    FireworkSdk.init(
      config,
      onSuccess = {
        FWEventUtils.sendInitSuccessEvent(methodChannel)
      },
      onError = { error ->
        FWEventUtils.sendInitFailedEvent(methodChannel, "$error")
      },
    )
  }

  private fun getAppid(): String? {
    val ai = context.packageManager.getApplicationInfo(
      context.packageName,
      PackageManager.GET_META_DATA
    )
    val value = ai.metaData.get("Firework:Appid")
    return value?.let {
      it as String
    } ?: run {
      null
    }
  }

  @SuppressLint("HardwareIds")
  private fun getHashedAndroidId(): String {
    val androidId = Settings.Secure.getString(
      context.contentResolver,
      Settings.Secure.ANDROID_ID
    )
    return getHash(androidId)
  }

  private fun getHash(text: String): String {
    val digest: MessageDigest = MessageDigest.getInstance("SHA-256")
    val hash: ByteArray = digest.digest(text.toByteArray(StandardCharsets.UTF_8))
    return Base64.encodeToString(hash, Base64.DEFAULT)
  }

  @FwAnalyticCallable
  fun onCtaButtonClicked(event: CtaButtonClickAnalyticsEvent) {
    if (FWGlobalDataUtil.videoPlaybackEventEnabled) {
      FWEventUtils.sendVideoPlaybackEvent(
        methodChannel,
        FWVideoPlaybackSubEventName.ClickCTA.rawValue,
        FWVideoPlaybackDetails(
          badge = event.videoInfo.badge,
          caption = event.videoInfo.caption,
          duration = event.videoInfo.duration,
          hasCta = true,
          playerHeight = event.videoInfo.playerHeight,
          playerWidth = event.videoInfo.playerWidth,
          id = event.videoInfo.id,
          actionUrl = event.actionUrl,
        )
      )
    }

    FWEventUtils.sendCustomCTAClickEvent(methodChannel, event.actionUrl)
  }

  @FwAnalyticCallable
  fun onShareButtonClicked(event: ShareButtonAnalyticsEvent) {
    if (FWGlobalDataUtil.videoPlaybackEventEnabled) {
      FWEventUtils.sendVideoPlaybackEvent(
        methodChannel,
        FWVideoPlaybackSubEventName.ClickShare.rawValue,
        FWVideoPlaybackDetails(
          badge = event.videoInfo.badge,
          caption = event.videoInfo.caption,
          duration = event.videoInfo.duration,
          hasCta = event.videoInfo.hasCta,
          playerHeight = event.videoInfo.playerHeight,
          playerWidth = event.videoInfo.playerWidth,
          id = event.videoInfo.id,
        )
      )
    }
  }

  @FwAnalyticCallable
  fun onPlaybackEvent(event: PlayerLifecycleAnalyticsEvent) {
    if (!FWGlobalDataUtil.videoPlaybackEventEnabled) {
      return
    }
    val progress = event.progress

    val videoPlaybackDetails = FWVideoPlaybackDetails(
      badge = event.videoInfo.badge,
      caption = event.videoInfo.caption,
      duration = event.videoInfo.duration,
      hasCta = event.videoInfo.hasCta,
      playerHeight = event.videoInfo.playerHeight,
      playerWidth = event.videoInfo.playerWidth,
      id = event.videoInfo.id,
      progress = progress
    )
    when (event) {
      is PlayerLifecycleAnalyticsEvent.OnStarted -> {
        if (event.videoInfo.isAd) {
          FWEventUtils.sendVideoPlaybackEvent(methodChannel, FWVideoPlaybackSubEventName.AdStart.rawValue, videoPlaybackDetails)
        } else {
          FWEventUtils.sendVideoPlaybackEvent(methodChannel, FWVideoPlaybackSubEventName.Start.rawValue, videoPlaybackDetails)
        }
      }
      is PlayerLifecycleAnalyticsEvent.OnPrepared -> {
        FWEventUtils.sendVideoPlaybackEvent(methodChannel, FWVideoPlaybackSubEventName.Impression.rawValue, videoPlaybackDetails)
      }
      is PlayerLifecycleAnalyticsEvent.OnBuffering -> {}
      is PlayerLifecycleAnalyticsEvent.OnIdle -> {}
      is PlayerLifecycleAnalyticsEvent.OnRepeat -> {}
      is PlayerLifecycleAnalyticsEvent.OnEnded -> {
        if (event.videoInfo.isAd) {
          FWEventUtils.sendVideoPlaybackEvent(methodChannel, FWVideoPlaybackSubEventName.AdEnd.rawValue, videoPlaybackDetails)
        } else {
          FWEventUtils.sendVideoPlaybackEvent(methodChannel, FWVideoPlaybackSubEventName.Complete.rawValue, videoPlaybackDetails)
        }
      }
      is PlayerLifecycleAnalyticsEvent.OnError -> {}
      is PlayerLifecycleAnalyticsEvent.OnSeek -> {}
      is PlayerLifecycleAnalyticsEvent.OnVideoSizeChanged -> {}
      is PlayerLifecycleAnalyticsEvent.OnFirstQuartile -> {
        FWEventUtils.sendVideoPlaybackEvent(methodChannel, FWVideoPlaybackSubEventName.FirstQuartile.rawValue, videoPlaybackDetails)
      }
      is PlayerLifecycleAnalyticsEvent.OnSecondQuartile -> {
        FWEventUtils.sendVideoPlaybackEvent(methodChannel, FWVideoPlaybackSubEventName.Midpoint.rawValue, videoPlaybackDetails)
      }
      is PlayerLifecycleAnalyticsEvent.OnThirdQuartile -> {
        FWEventUtils.sendVideoPlaybackEvent(methodChannel, FWVideoPlaybackSubEventName.ThirdQuartile.rawValue, videoPlaybackDetails)
      }
      is PlayerLifecycleAnalyticsEvent.OnPlaybackProgressChanged -> {}
      is PlayerLifecycleAnalyticsEvent.OnPaused -> {
        FWEventUtils.sendVideoPlaybackEvent(methodChannel, FWVideoPlaybackSubEventName.Pause.rawValue, videoPlaybackDetails)
      }
      is PlayerLifecycleAnalyticsEvent.OnResumed -> {
        FWEventUtils.sendVideoPlaybackEvent(methodChannel, FWVideoPlaybackSubEventName.Resume.rawValue, videoPlaybackDetails)
      }
      else -> {}
    }
  }
}