package com.fireworksdk.bridge.flutter.platformview

import android.app.Activity
import android.view.View
import android.view.ViewGroup
import android.widget.FrameLayout
import com.firework.videofeed.FeedViewState
import com.fireworksdk.bridge.components.videofeed.FWVideoFeed
import com.fireworksdk.bridge.flutter.handler.FWVideoFeedHandler
import com.fireworksdk.bridge.flutter.models.FWChannelName
import com.fireworksdk.bridge.flutter.utils.FWEventUtils
import com.fireworksdk.bridge.models.*
import com.fireworksdk.bridge.utils.FWCommonUtil
import com.fireworksdk.bridge.utils.FWLogUtils
import io.flutter.plugin.common.BinaryMessenger
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.platform.PlatformView
import org.json.JSONObject

class FWVideoFeedPlatformView(
  activity: Activity,
  id: Int,
  creationParams: Map<String?, Any?>?,
  binaryMessenger: BinaryMessenger,
  fwSdkModuleChannel: MethodChannel?,
) : PlatformView {

  val videoFeed: FWVideoFeed = FWVideoFeed(activity)
  private val methodChannel = MethodChannel(
    binaryMessenger,
    FWChannelName.FireworkVideoFeed.rawValue.replace("(viewId)", "$id")
  )

  private val fwSdkModuleChannel by weakProperty<MethodChannel?>(fwSdkModuleChannel)

  init {
    methodChannel.setMethodCallHandler(FWVideoFeedHandler(this))
    val creationParamsMutableMap = creationParams?.toMutableMap()
    val creationParamsJSONObject = creationParamsMutableMap?.let {
      JSONObject(it)
    }
    val videoFeedProps: FWVideoFeedPropsModel? =
      FWVideoFeedPropsModelDeserializer.deserialize(creationParamsJSONObject)
    setProps(videoFeedProps)
    addVideoFeedListener()
    addPlaylistGroupFeedListener()
  }

  private fun setProps(props: FWVideoFeedPropsModel?) {
    props?.let {
      videoFeed.videoFeedPropsModel = it
      videoFeed.initVideoFeedView()
    }

    val right = props?.marginRight
    if (right != null && right > 0) {
      val layoutParams = FrameLayout.LayoutParams(
        ViewGroup.LayoutParams.MATCH_PARENT,
        ViewGroup.LayoutParams.MATCH_PARENT
      )
      layoutParams.setMargins(0, 0, FWCommonUtil.dpToPx(right.toFloat(), videoFeed.context), 0)
      videoFeed.setVideoFeedLayoutParams(layoutParams)
    }
  }

  private fun addVideoFeedListener() {
    val videoFeedView = videoFeed.videoFeedView
    videoFeedView.setOnFeedItemClickListener { feedItem ->
      val index = feedItem.indexInTheList
      val id = feedItem.id
      val duration = feedItem.duration
      val title = feedItem.title

      val fwVideoFeedItemDetailsModel = FWVideoFeedItemDetailsModel(
        index,
        id,
        duration.toInt(),
        videoFeed.videoFeedPropsModel.source?.rawValue,
        title,
        videoFeed.videoFeedPropsModel.playlistGroup,
        videoFeed.videoFeedPropsModel.playlist,
        videoFeed.videoFeedPropsModel.channel,
        videoFeed.videoFeedPropsModel.dynamicContentParameters
      )
      FWEventUtils.sendVideoFeedClickEvent(fwSdkModuleChannel, fwVideoFeedItemDetailsModel)
    }

    videoFeedView.setOnFeedViewStateListener { feedViewState ->
      when (feedViewState) {
        is FeedViewState.Loading -> {
          FWLogUtils.d { "FeedViewState.Loading" }
        }
        is FeedViewState.LoadData -> {
          FWLogUtils.d { "FeedViewState.LoadData" }
          FWEventUtils.sendVideoFeedLoadSuccessEvent(methodChannel)
        }
        is FeedViewState.ScrollToPosition -> {
          FWLogUtils.d { "FeedViewState.ScrollToPosition" }
        }
        is FeedViewState.OpenFeedElementAtIndex -> {
          FWLogUtils.d { "FeedViewState.OpenFeedElementAtIndex" }
        }
        is FeedViewState.Error -> {
          FWLogUtils.d { "FeedViewState.Error" }
          FWEventUtils.sendVideoFeedLoadFailedEvent(methodChannel, feedViewState.message, feedViewState.message)
        }
        else -> {}
      }}
  }

  private fun addPlaylistGroupFeedListener() {

  }

  override fun getView(): View {
    return videoFeed
  }

  override fun dispose() {
    videoFeed.videoFeedView.destroy()
  }
}