package com.fireworksdk.bridge.utils

import android.content.Context
import android.graphics.Typeface
import android.graphics.drawable.GradientDrawable
import com.firework.common.PlayerMode
import com.firework.common.ad.AdBadgeOption
import com.firework.common.ad.AdBadgeTextType
import com.firework.common.ad.AdOption
import com.firework.common.cta.CtaDelay
import com.firework.common.cta.CtaDelayUnit
import com.firework.common.feed.FeedLayout
import com.firework.common.feed.FeedResource
import com.firework.common.feed.FeedTitlePosition
import com.firework.viewoptions.*
import com.fireworksdk.bridge.FWInitializationProvider
import com.fireworksdk.bridge.models.*
import com.fireworksdk.bridge.models.enums.FWBadgeTextType
import com.fireworksdk.bridge.models.enums.FWCtaDelayType
import com.fireworksdk.bridge.models.enums.FWGradientDrawableOrientation
import com.fireworksdk.bridge.models.enums.FWPlayerStyle
import com.fireworksdk.bridge.models.enums.FWSystemTypeface
import com.fireworksdk.bridge.models.enums.FWVideoCompleteAction
import com.fireworksdk.bridge.models.enums.FWVideoFeedMode
import com.fireworksdk.bridge.models.enums.FWVideoFeedSource
import com.fireworksdk.bridge.models.enums.FWVideoFeedTitlePosition
import com.fireworksdk.bridge.models.enums.FWVideoPlayerCTAWidth

object FWConfigUtil {

  fun generateViewOptionsBuilder(context: Context?, videoFeedPropsModel: FWVideoFeedPropsModel?): ViewOptions.Builder {
    context ?: return ViewOptions.Builder()
    videoFeedPropsModel ?: return ViewOptions.Builder()

    val layoutOptionBuilder = getDefaultLayoutOptionBuilder()
    val titleOptionBuilder = getDefaultTitleOptionBuilder()
    val playerOptionBuilder = getDefaultPlayerOptionBuilder()
    val baseOptionBuilder = getDefaultBaseOptionBuilder()
    val adBadgeOptionBuilder = getDefaultAdBadgeOptionBuilder()
    val ctaOptionBuilder = getDefaultCtaOptionBuilder()
    val adOptionBuilder = getDefaultAdOptionBuilder()

    when (videoFeedPropsModel.source) {
      FWVideoFeedSource.Discover -> {
        baseOptionBuilder.feedResource(FeedResource.Discovery)
      }
      FWVideoFeedSource.Channel -> {
        videoFeedPropsModel.channel?.let { channel ->
          baseOptionBuilder.feedResource(FeedResource.Channel(channel))
        }
      }
      FWVideoFeedSource.Playlist -> {
        videoFeedPropsModel.channel?.let { channel ->
          videoFeedPropsModel.playlist?.let { playlist ->
            baseOptionBuilder.feedResource(FeedResource.Playlist(channel, playlist))
          }
        }
      }
      FWVideoFeedSource.DynamicContent -> {
        videoFeedPropsModel.channel?.let { channel ->
          baseOptionBuilder.feedResource(FeedResource.DynamicContent(channel, videoFeedPropsModel.dynamicContentParameters?:hashMapOf()))
        }
      }
      FWVideoFeedSource.PlaylistGroup -> {

      }
      FWVideoFeedSource.HashtagPlaylist -> {
        videoFeedPropsModel.channel?.let { channel ->
          if (!videoFeedPropsModel.hashtagFilterExpression.isNullOrBlank()) {
            baseOptionBuilder.feedResource(FeedResource.ChannelHashtag(channel, videoFeedPropsModel.hashtagFilterExpression))
          } else {
            baseOptionBuilder.feedResource(FeedResource.Channel(channel))
          }
        }
      }
      FWVideoFeedSource.Sku -> {
        videoFeedPropsModel.channel?.let { channel ->
          baseOptionBuilder.feedResource(FeedResource.Sku(channel, videoFeedPropsModel.productIds ?: emptyList()))
        }
      }
      FWVideoFeedSource.ShareUrl -> {
        videoFeedPropsModel.shareUrl?.let { shareUrl ->
          baseOptionBuilder.feedResource(FeedResource.ShareUrl(shareUrl))
        }
      }
      FWVideoFeedSource.SingleContent -> {
        videoFeedPropsModel.contentId?.let { contentId ->
          baseOptionBuilder.feedResource(FeedResource.SingleContent(contentId))
        }
      }
      else -> {}
    }

    when (videoFeedPropsModel.mode) {
      FWVideoFeedMode.Row -> {
        layoutOptionBuilder.feedLayout(FeedLayout.HORIZONTAL)
      }
      FWVideoFeedMode.Column -> {
        layoutOptionBuilder.feedLayout(FeedLayout.VERTICAL)
      }
      FWVideoFeedMode.Grid -> {
        layoutOptionBuilder.feedLayout(FeedLayout.GRID)
      }
      else -> {}
    }

    val backgroundColor = videoFeedPropsModel.videoFeedConfiguration?.backgroundColor
    if (!backgroundColor.isNullOrBlank()) {
      layoutOptionBuilder.backgroundColor(FWColorUtil.parseColor(backgroundColor))
    }

    val cornerRadius = videoFeedPropsModel.videoFeedConfiguration?.cornerRadius
    if (cornerRadius != null && cornerRadius >= 0) {
      layoutOptionBuilder.roundedCorner(true)
        .roundedCornerRadius(FWCommonUtil.dpToPx(cornerRadius.toFloat(), context))
    }

    val title = videoFeedPropsModel.videoFeedConfiguration?.title
    if (title != null) {
      val titleHidden = title.hidden
      val titleTextColor = title.textColor
      val titleFontSize = title.fontSize
      val titleBackgroundColor = title.backgroundColor
      val titleGradientDrawable = title.gradientDrawable
      val typeface = getTypeface(context, title.fontInfo)
      if (titleHidden != null) {
        titleOptionBuilder.showFeedTitle(!titleHidden)
      }
      if (!titleTextColor.isNullOrBlank()) {
        titleOptionBuilder.feedTitleTextColor(FWColorUtil.parseColor(titleTextColor))
      }
      if (titleFontSize != null && titleFontSize > 0) {
        titleOptionBuilder.feedTitleTextSize(FWCommonUtil.spToPx(titleFontSize.toFloat(), context))
      }
      typeface?.let {
        titleOptionBuilder.feedTitleTextTypeface(it)
      }
      if (!titleBackgroundColor.isNullOrBlank()) {
        titleOptionBuilder.feedTitleBackgroundColor(FWColorUtil.parseColor(titleBackgroundColor))
      }
      if (titleGradientDrawable != null) {
        val orientation = titleGradientDrawable.orientation
        val colors = titleGradientDrawable.colors
        if (orientation != null && !colors.isNullOrEmpty()) {
          var gradientDrawableOrientation = GradientDrawable.Orientation.LEFT_RIGHT
          when {
            orientation === FWGradientDrawableOrientation.TOP_BOTTOM -> {
              gradientDrawableOrientation = GradientDrawable.Orientation.TOP_BOTTOM
            }
            orientation === FWGradientDrawableOrientation.TR_BL -> {
              gradientDrawableOrientation = GradientDrawable.Orientation.TR_BL
            }
            orientation === FWGradientDrawableOrientation.RIGHT_LEFT -> {
              gradientDrawableOrientation = GradientDrawable.Orientation.RIGHT_LEFT
            }
            orientation === FWGradientDrawableOrientation.BR_TL -> {
              gradientDrawableOrientation = GradientDrawable.Orientation.BR_TL
            }
            orientation === FWGradientDrawableOrientation.BOTTOM_TOP -> {
              gradientDrawableOrientation = GradientDrawable.Orientation.BOTTOM_TOP
            }
            orientation === FWGradientDrawableOrientation.BL_TR -> {
              gradientDrawableOrientation = GradientDrawable.Orientation.BL_TR
            }
            orientation === FWGradientDrawableOrientation.LEFT_RIGHT -> {
              gradientDrawableOrientation = GradientDrawable.Orientation.BL_TR
            }
            orientation === FWGradientDrawableOrientation.TL_BR -> {
              gradientDrawableOrientation = GradientDrawable.Orientation.TL_BR
            }
            else -> {}
          }
          val intColors = colors.map {
            FWColorUtil.parseColor(it)
          }
          titleOptionBuilder.feedTitleBackgroundDrawable(
            GradientDrawable(
              gradientDrawableOrientation,
              intColors.toIntArray(),
            )
          )
        }
      }
    }

    when (videoFeedPropsModel.videoFeedConfiguration?.titlePosition) {
      FWVideoFeedTitlePosition.Nested -> {
        layoutOptionBuilder.feedTitlePosition(FeedTitlePosition.NESTED)
      }
      FWVideoFeedTitlePosition.Stacked -> {
        layoutOptionBuilder.feedTitlePosition(FeedTitlePosition.STACKED)
      }
      else -> {}
    }

    val playIcon = videoFeedPropsModel.videoFeedConfiguration?.playIcon
    if (playIcon != null) {
      val hidden = playIcon.hidden
      val iconWidth = playIcon.iconWidth
      layoutOptionBuilder.showPlayIcon(hidden != true)
      if (iconWidth != null && iconWidth >= 0) {
        layoutOptionBuilder.playIconWidth(FWCommonUtil.dpToPx(iconWidth.toFloat(), context))
      }
    }

    val showAdBadge = videoFeedPropsModel.videoFeedConfiguration?.showAdBadge
    if (showAdBadge != null) {
      adBadgeOptionBuilder.adBadgeShowOnThumbnails(showAdBadge)
    }

    val adBadgeConfigModel = FWGlobalDataUtil.adBadgeConfigModel
    val adBadgeTextType = adBadgeConfigModel.badgeTextType
    val adBadgeBackgroundColor = adBadgeConfigModel.backgroundColor
    val adBadgeTextColor = adBadgeConfigModel.textColor
    val adBadgeTypeface = getTypeface(context, adBadgeConfigModel.fontInfo)
    when (adBadgeTextType) {
      FWBadgeTextType.Ad -> {
        adBadgeOptionBuilder.adBadgeLabel(AdBadgeTextType.AD)
      }
      FWBadgeTextType.Sponsored -> {
        adBadgeOptionBuilder.adBadgeLabel(AdBadgeTextType.SPONSORED)
      }
      else -> {}
    }
    if (!adBadgeBackgroundColor.isNullOrBlank()) {
      adBadgeOptionBuilder.adBadgeBackColor(FWColorUtil.parseColor(adBadgeBackgroundColor))
    }
    if (!adBadgeTextColor.isNullOrBlank()) {
      adBadgeOptionBuilder.adBadgeTextColor(FWColorUtil.parseColor(adBadgeTextColor))
    }
    adBadgeTypeface?.let {
      adBadgeOptionBuilder.adBadgeTypeface(it)
    }

    val enableAutoplay = videoFeedPropsModel.videoFeedConfiguration?.enableAutoplay
    enableAutoplay?.let {
      playerOptionBuilder.autoplay(it)
    }

    FWGlobalDataUtil.shareBaseUrl?.let {
      playerOptionBuilder.shareBaseUrl(it)
    }

    val shareBaseUrl = videoFeedPropsModel.videoPlayerConfiguration?.shareBaseUrl
    shareBaseUrl?.let {
      playerOptionBuilder.shareBaseUrl(it)
    }

    playerOptionBuilder.sdkHandleCtaButtonClick(!FWGlobalDataUtil.customCTAClickEnabled)

    val gridColumns = videoFeedPropsModel.videoFeedConfiguration?.gridColumns
    if (gridColumns != null && gridColumns > 0) {
      layoutOptionBuilder.columnCount(gridColumns)
    }

    val itemSpacing = videoFeedPropsModel.videoFeedConfiguration?.itemSpacing
    if (itemSpacing != null && itemSpacing >= 0) {
      layoutOptionBuilder.itemSpacing(FWCommonUtil.dpToPx(itemSpacing.toFloat(), context) / 2)
    }

    // player config
    when (videoFeedPropsModel.videoPlayerConfiguration?.playerStyle) {
      FWPlayerStyle.Fit -> {
        playerOptionBuilder.playerMode(PlayerMode.FIT_MODE)
      }
      FWPlayerStyle.Full -> {
        playerOptionBuilder.playerMode(PlayerMode.FULL_BLEED_MODE)
      }
      else -> {}
    }

    when (videoFeedPropsModel.videoPlayerConfiguration?.videoCompleteAction) {
      FWVideoCompleteAction.Loop -> {
        playerOptionBuilder.autoPlayOnComplete(false)
      }
      FWVideoCompleteAction.AdvanceToNext -> {
        playerOptionBuilder.autoPlayOnComplete(true)
      }
      else -> {}
    }

    val showShareButton = videoFeedPropsModel.videoPlayerConfiguration?.showShareButton
    showShareButton?.let {
      playerOptionBuilder.showShareButton(it)
    }

    val enablePipMode = videoFeedPropsModel.videoPlayerConfiguration?.enablePictureInPicture
    enablePipMode?.let {
      playerOptionBuilder.enablePipMode(it)
    }

    when (videoFeedPropsModel.videoPlayerConfiguration?.ctaWidth) {
      FWVideoPlayerCTAWidth.FullWidth -> {
        ctaOptionBuilder.ctaMode(CtaOption.CtaMode.FULL_WIDTH)
      }
      FWVideoPlayerCTAWidth.Compact -> {
        ctaOptionBuilder.ctaMode(CtaOption.CtaMode.COMPACT)
      }
      FWVideoPlayerCTAWidth.SizeToFit -> {
        ctaOptionBuilder.ctaMode(CtaOption.CtaMode.SIZE_TO_FIT)
      }
      else -> {}
    }

    val ctaDelay = videoFeedPropsModel.videoPlayerConfiguration?.ctaDelay
    ctaDelay?.let { delay ->
      delay.value ?: return@let

      when (delay.type) {
        FWCtaDelayType.Constant -> {
          ctaOptionBuilder.ctaDelay(
            CtaDelay(delay.value.toFloat(), CtaDelayUnit.SECONDS)
          )
        }
        FWCtaDelayType.Percentage -> {
          ctaOptionBuilder.ctaDelay(
            CtaDelay(delay.value.toFloat(), CtaDelayUnit.PERCENTAGE)
          )
        }
        else -> {}
      }
    }

    val ctaHighlightDelay = videoFeedPropsModel.videoPlayerConfiguration?.ctaHighlightDelay
    ctaHighlightDelay?.let { delay ->
      delay.value ?: return@let

      when (delay.type) {
        FWCtaDelayType.Constant -> {
          ctaOptionBuilder.ctaHighlightDelay(
            CtaDelay(delay.value.toFloat(), CtaDelayUnit.SECONDS)
          )
        }
        FWCtaDelayType.Percentage -> {
          ctaOptionBuilder.ctaHighlightDelay(
            CtaDelay(delay.value.toFloat(), CtaDelayUnit.PERCENTAGE)
          )
        }
        else -> {}
      }
    }

    val showMuteButton = videoFeedPropsModel.videoPlayerConfiguration?.showMuteButton
    showMuteButton?.let {
      playerOptionBuilder.showMuteButton(it)
    }

    val showPlaybackButton = videoFeedPropsModel.videoPlayerConfiguration?.showPlaybackButton
    showPlaybackButton?.let {
      playerOptionBuilder.showPlayPauseButtonInReplay(it)
    }

    val showBranding = videoFeedPropsModel.videoPlayerConfiguration?.showBranding
    showBranding?.let {
      playerOptionBuilder.showFireworkLogo(it)
    }

    return ViewOptions.Builder()
      .layoutOption(layoutOptionBuilder.build())
      .titleOption(titleOptionBuilder.build())
      .playerOption(playerOptionBuilder.build())
      .baseOption(baseOptionBuilder.build())
      .adBadgeOption(adBadgeOptionBuilder.build())
      .ctaOption(ctaOptionBuilder.build())
      .adOption(adOptionBuilder.build())
  }

  private fun getDefaultLayoutOptionBuilder(): LayoutOption.Builder {
    val context = FWInitializationProvider.INSTANCE.resumedActivity ?: return LayoutOption.Builder()
    return LayoutOption.Builder()
      .playIconWidth(FWCommonUtil.dpToPx("36".toFloat(), context))
  }

  private fun getDefaultTitleOptionBuilder(): TitleOption.Builder {
    return TitleOption.Builder()
  }

  private fun getDefaultPlayerOptionBuilder(): PlayerOption.Builder {
    return PlayerOption.Builder()
      .enablePipMode(false)
  }

  private fun getDefaultBaseOptionBuilder(): BaseOption.Builder {
    return BaseOption.Builder()
  }

  private fun getDefaultAdBadgeOptionBuilder(): AdBadgeOption.Builder {
    return AdBadgeOption.Builder()
  }

  private fun getDefaultCtaOptionBuilder(): CtaOption.Builder {
    return CtaOption.Builder()
  }

  private fun getDefaultAdOptionBuilder(): AdOption.Builder {
    return AdOption.Builder()
  }

  private fun getTypeface(context: Context, font: FWFontInfoModel?): Typeface? {
    val typefaceName = font?.typefaceName
    typefaceName ?: return null

    var typeface = Typeface.DEFAULT
    if (font.isCustom == true) {
      typeface = Typeface.createFromAsset(context.assets, typefaceName.rawValue)
    } else {
      when (typefaceName) {
        FWSystemTypeface.DefaultBold -> {
          typeface = Typeface.DEFAULT_BOLD
        }
        FWSystemTypeface.SansSerif -> {
          typeface = Typeface.SANS_SERIF
        }
        FWSystemTypeface.Serif -> {
          typeface = Typeface.SERIF
        }
        FWSystemTypeface.Monospace -> {
          typeface = Typeface.MONOSPACE
        }
        else -> {}
      }
    }
    return typeface
  }

}
