//
//  StoryBlock.swift
//  fw_flutter_sdk
//
//  Created by linjie jiang on 2022/8/10.
//

import FireworkVideo
import UIKit

@objc
public enum StoryBlockSourceType: Int {
    case discover, channel, playlist, dynamicContent, hashtagPlaylist, sku, singleContent
}

public protocol StoryBlockDelegate: AnyObject {
    func storyBlockDidLoadFeed(_ view: StoryBlock)
    func storyBlock(_ view: StoryBlock, didFailToLoadFeed error: StoryBlockError)
    func storyBlock(_ view: StoryBlock, pdpVisibilityChange visible: Bool)
}

@objc
public class StoryBlock: UIView, StoryBlockViewControllerDelegate {
    @objc public var sourceType: StoryBlockSourceType = .discover
    @objc public var channel: String = ""
    @objc public var playlist: String = ""
    @objc public var dynamicContentParameters: [String: [String]] = [:]
    @objc public var hashtagFilterExpression: String = ""
    @objc public var productIds: [String] = []
    @objc public var contentId: String = ""
    @objc public var adConfiguration: AdConfiguration?
    @objc public var storyBlockConfiguration: StoryBlockConfiguration? {
        didSet {
            updateStoryBlockVCConfiguration()
        }
    }

    public weak var delegate: StoryBlockDelegate?

    private var storyBlockVC: StoryBlockViewController?

    private var source: StoryBlockContentSource {
        switch sourceType {
        case .discover:
            return .discover
        case .channel:
            return .channel(channelID: channel)
        case .playlist:
            return .channelPlaylist(channelID: channel, playlistID: playlist)
        case .dynamicContent:
            return .dynamicContent(channelID: channel, parameters: dynamicContentParameters)
        case .hashtagPlaylist:
            return .hashtagPlaylist(channelID: channel, filterExpression: hashtagFilterExpression)
        case .sku:
            return .skuPlaylist(channelID: channel, productIDs: productIds)
        case .singleContent:
            return .singleContent(videoOrLiveStreamID: contentId)
        }
    }

    public override init(frame: CGRect) {
        super.init(frame: frame)
        self.clipsToBounds = true
    }

    required init?(coder: NSCoder) {
        super.init(coder: coder)
        self.clipsToBounds = true
    }

    public override func layoutSubviews() {
        super.layoutSubviews()

        embed()
    }

    private func embed() {
        guard let parentVC = parentViewController else {
            return
        }
        guard self.storyBlockVC == nil else {
            return
        }

        var resultStoryBlockVC: StoryBlockViewController?
        if let fireworkVideoAdConfiguration = Convert.getFireworkVideoAdConfiguration(adConfiguration) {
            resultStoryBlockVC = StoryBlockViewController(source: source, adConfiguration: fireworkVideoAdConfiguration)
        } else {
            resultStoryBlockVC = StoryBlockViewController(source: source)
        }

        guard let storyBlockVC = resultStoryBlockVC else {
            return
        }

        storyBlockVC.isPictureInPictureEnabled = storyBlockConfiguration?.enablePictureInPicture ?? true

        storyBlockVC.delegate = self
        self.storyBlockVC = storyBlockVC
        updateStoryBlockVCConfiguration()
        parentVC.attachChild(storyBlockVC, to: self)
    }

    public override func removeFromSuperview() {
        super.removeFromSuperview()

        guard let storyBlockVC = self.storyBlockVC else {
            return
        }

        storyBlockVC.detachFromParent()
        self.storyBlockVC = nil
    }

    public func play() {
        guard let storyBlockVC = self.storyBlockVC else {
            return
        }
        storyBlockVC.play()
    }

    public func pause() {
        guard let storyBlockVC = self.storyBlockVC else {
            return
        }
        storyBlockVC.pause()
    }

    public func storyBlockDidLoadFeed(_ viewController: StoryBlockViewController) {
        guard let delegate = self.delegate else {
            return
        }

        delegate.storyBlockDidLoadFeed(self)
    }

    public func storyBlock(
        _ viewController: StoryBlockViewController, didFailToLoadFeed error: StoryBlockError
    ) {
        guard let delegate = self.delegate else {
            return
        }

        delegate.storyBlock(self, didFailToLoadFeed: error)
    }

    public func notifyPdpVisibilityChange(_ visible: Bool) {
        guard let delegate = self.delegate else {
            return
        }

        delegate.storyBlock(self, pdpVisibilityChange: visible)
    }

    // swiftlint:disable:next cyclomatic_complexity function_body_length
    private func updateStoryBlockVCConfiguration() {
        guard let storyBlockVC = storyBlockVC else {
            return
        }

        var resultConfig = FireworkVideo.StoryBlockConfiguration()
        guard let config = storyBlockConfiguration else {
            storyBlockVC.viewConfiguration = resultConfig
            return
        }

        if let videoCompleteAction = config.videoCompleteAction {
            switch videoCompleteAction {
            case .loop:
                resultConfig.fullScreenPlayerView.videoCompleteAction = .loop
            default:
                resultConfig.fullScreenPlayerView.videoCompleteAction = .advanceToNext
            }
        }

        if let showShareButton = config.showShareButton {
            resultConfig.shareButton.isHidden = !showShareButton
            resultConfig.fullScreenPlayerView.shareButton.isHidden = !showShareButton
        }

        if let showBranding = config.showBranding {
            resultConfig.videoDetail.fireworkAttribution.isHidden = !showBranding
            resultConfig.fullScreenPlayerView.videoDetail.fireworkAttribution.isHidden = !showBranding
        }

        if let showVideoDetailTitle = config.showVideoDetailTitle {
            resultConfig.videoDetail.titleConfiguration.isHidden = !showVideoDetailTitle
            resultConfig.fullScreenPlayerView.videoDetail.titleConfiguration.isHidden = !showVideoDetailTitle
        }

        if let shareBaseURL = config.shareBaseURL {
            resultConfig.shareButton.behavior.baseURL = URL(string: shareBaseURL)
            resultConfig.fullScreenPlayerView.shareButton.behavior.baseURL = URL(string: shareBaseURL)
        } else if let shareBaseUrl = FireworkSDKPlugin.shareBaseURL {
            resultConfig.shareButton.behavior.baseURL = URL(string: shareBaseUrl)
            resultConfig.fullScreenPlayerView.shareButton.behavior.baseURL = URL(string: shareBaseUrl)
        }

        if let ctaWidth = config.ctaWidth {
            resultConfig.ctaButton.width = ctaWidth.width()
            resultConfig.fullScreenPlayerView.ctaButton.width = ctaWidth.width()
        }

        if let ctaDelay = config.ctaDelay,
           let type = ctaDelay.type,
           let value = ctaDelay.value {
            switch type {
            case .constant:
                resultConfig.ctaButton.behavior.delay = .constant(value)
                resultConfig.fullScreenPlayerView.ctaButton.behavior.delay = .constant(value)
            case .percentage:
                resultConfig.ctaButton.behavior.delay = .percentage(Float(value))
                resultConfig.fullScreenPlayerView.ctaButton.behavior.delay = .percentage(Float(value))
            }
        }

        if let ctaHighlightDelay = config.ctaHighlightDelay,
           let type = ctaHighlightDelay.type,
           let value = ctaHighlightDelay.value {
            switch type {
            case .constant:
                resultConfig.ctaButton.behavior.highlightDelay = .constant(value)
                resultConfig.fullScreenPlayerView.ctaButton.behavior.highlightDelay = .constant(value)
            case .percentage:
                resultConfig.ctaButton.behavior.highlightDelay = .percentage(Float(value))
                resultConfig.fullScreenPlayerView.ctaButton.behavior.highlightDelay = .percentage(Float(value))
            }
        }

        if let adBadge = FireworkSDKPlugin.convertToFWAdBadgeConfiguration(FireworkSDKPlugin.adBadgeConfiguration) {
            resultConfig.adBadge = adBadge
        }

        storyBlockVC.viewConfiguration = resultConfig
    }
}
