//
//  FWVideoFeed.swift
//  fw_flutter_sdk
//
//  Created by Jeff Zheng on 2021/12/22.
//

// swiftlint:disable file_length

import FireworkVideo
import UIKit

let gOnVideoTemporarilyReleasePipController = Notification.Name("gOnVideoTemporarilyReleasePipController")
let gOnVideoRestorePipController = Notification.Name("gOnVideoRestorePipController")

@objc
public enum VideoFeedMode: Int {
    case row, column, grid
}

@objc
public enum VideFeedSourceType: Int {
    case discover, channel, playlist, playlistGroup, dynamicContent, hashtagPlaylist, sku, singleContent
}

public protocol VideoFeedDelegate: AnyObject {
    /// Called once after a video feed has been successfully loaded
    func videoFeedDidLoadFeed(_ view: VideoFeed)
    /// Called if the video feed failed to load
    func videoFeed(
        _ view: VideoFeed,
        didFailToLoadFeed error: VideoFeedError)
}

@objc
public class VideoFeed: UIView, VideoFeedViewControllerDelegate {
    @objc public var sourceType: VideFeedSourceType = .discover
    @objc public var channel: String = ""
    @objc public var playlist: String = ""
    @objc public var playlistGroup: String = ""
    @objc public var dynamicContentParameters: [String: [String]] = [:]
    @objc public var hashtagFilterExpression: String = ""
    @objc public var productIds: [String] = []
    @objc public var contentId: String = ""

    @objc public var mode: VideoFeedMode = .row
    @objc public var feedViewConfig: VideoFeedConfiguration? {
        didSet {
            guard let feedVC = feedVC else {
                return
            }
            feedVC.viewConfiguration = convertToVideoFeedContentConfiguration()
            feedVC.layout = videoFeedLayout
        }
    }
    @objc public var playerViewConfig: VideoPlayerConfiguration? {
        didSet {
            guard let feedVC = feedVC else {
                return
            }
            feedVC.viewConfiguration = convertToVideoFeedContentConfiguration()
        }
    }
    @objc public var adConfiguration: AdConfiguration?

    public weak var delegate: VideoFeedDelegate?

    private var feedVC: VideoFeedViewController?
    private var source: VideoFeedContentSource {
        switch sourceType {
        case .discover:
            return .discover
        case .channel:
            return .channel(channelID: channel)
        case .playlist:
            return .channelPlaylist(channelID: channel, playlistID: playlist)
        case .playlistGroup:
            return .playlistGroup(groupID: playlistGroup)
        case .dynamicContent:
            return .dynamicContent(channelID: channel, parameters: dynamicContentParameters)
        case .hashtagPlaylist:
            return .hashtagPlaylist(channelID: channel, filterExpression: hashtagFilterExpression)
        case .sku:
            return .skuPlaylist(channelID: channel, productIDs: productIds)
        case .singleContent:
            return .singleContent(videoOrLiveStreamID: contentId)
        }
    }

    private var videoFeedLayout: VideoFeedLayout {
        var resultLayout: VideoFeedLayout?
        switch mode {
        case .row:
            let layout = VideoFeedHorizontalLayout()
            resultLayout = layout
        case .column:
            let layout = VideoFeedGridLayout()
            layout.numberOfColumns = 1
            resultLayout = layout
        case .grid:
            let layout = VideoFeedGridLayout()
            if let gridColumns = feedViewConfig?.gridColumns,
                gridColumns > 0 {
                layout.numberOfColumns = gridColumns
            } else {
                layout.numberOfColumns = 2
            }
            resultLayout = layout
        }
        if let config = feedViewConfig {
            if let aspectRatio = config.aspectRatio {
                if let horizontalLayout = resultLayout as? VideoFeedHorizontalLayout {
                    horizontalLayout.itemWidthRatio = aspectRatio
                } else if let gridLayout = resultLayout as? VideoFeedGridLayout {
                    gridLayout.itemWidthRatio = aspectRatio
                }
            }
            if let contentPadding = config.contentPadding {
                if let top = contentPadding.top {
                    resultLayout?.contentInsets.top = top
                }
                if let right = contentPadding.right {
                    resultLayout?.contentInsets.right = right
                }
                if let bottom = contentPadding.bottom {
                    resultLayout?.contentInsets.bottom = bottom
                }
                if let left = contentPadding.left {
                    resultLayout?.contentInsets.left = left
                }
            }

            if let itemSpacing = config.itemSpacing {
                resultLayout?.itemSpacing = itemSpacing
            }
        }

        return resultLayout ?? VideoFeedHorizontalLayout()
    }

    public override func layoutSubviews() {
        super.layoutSubviews()

        embed()
    }

    private func embed() {
        guard let parentVC = parentViewController else {
            return
        }
        guard self.feedVC == nil else {
            return
        }

        var resultFeedVC: VideoFeedViewController?
        if let fireworkVideoAdConfiguration = Convert.getFireworkVideoAdConfiguration(adConfiguration) {
            resultFeedVC = VideoFeedViewController(
                layout: videoFeedLayout,
                source: source,
                adConfiguration: fireworkVideoAdConfiguration
            )
        } else {
            resultFeedVC = VideoFeedViewController(
                layout: videoFeedLayout,
                source: source
            )
        }

        guard let feedVC = resultFeedVC else {
            return
        }

        if let subView = feedVC.view.subviews.first {
            let constraints = feedVC.view.constraints
            for constraint in constraints {
                if (constraint.firstItem as? NSObject) == subView || (constraint.secondItem as? NSObject) == subView {
                    constraint.isActive = false
                }
            }
            NSLayoutConstraint.activate(subView.constraints(equalTo: feedVC.view))
        }

        var viewConfiguration = convertToVideoFeedContentConfiguration()
        if viewConfiguration.itemView.autoplay.isEnabled {
            viewConfiguration.itemView.autoplay.isEnabled = false
            feedVC.viewConfiguration = viewConfiguration
            DispatchQueue.main.async {
                viewConfiguration.itemView.autoplay.isEnabled = true
                feedVC.viewConfiguration = viewConfiguration
            }
        } else {
            feedVC.viewConfiguration = viewConfiguration
        }

        feedVC.isPictureInPictureEnabled = playerViewConfig?.enablePictureInPicture ?? true

        feedVC.delegate = self
        self.feedVC = feedVC

        parentVC.attachChild(feedVC, to: self)
    }

    public override func removeFromSuperview() {
        super.removeFromSuperview()

        guard let feedVC = self.feedVC else {
            return
        }

        feedVC.detachFromParent()
        self.feedVC = nil
    }

    @objc
    public func refresh() {
        guard let feedVC = self.feedVC else {
            return
        }
        feedVC.refresh()
    }

    // MARK: - VideoFeedViewControllerDelegate
    public func videoFeedDidLoadFeed(_ viewController: VideoFeedViewController) {
        guard let delegate = self.delegate else {
            return
        }
        delegate.videoFeedDidLoadFeed(self)
    }

    public func videoFeed(
        _ viewController: VideoFeedViewController, didFailToLoadFeed error: VideoFeedError
    ) {
        guard let delegate = self.delegate else {
            return
        }
        delegate.videoFeed(self, didFailToLoadFeed: error)
    }
}

extension VideoFeed {
    // swiftlint:disable:next cyclomatic_complexity function_body_length
    private static func convertToVideoFeedItemContentConfiguration(
        _ config: VideoFeedConfiguration?
    ) -> VideoFeedItemContentConfiguration? {
        guard let config = config else {
            return nil
        }

        var vfcConfig = VideoFeedItemContentConfiguration()
        // set default value so that behavior will the same with Android
        vfcConfig.title.isHidden = false
        vfcConfig.titleLayoutConfiguration.titlePosition = .nested

        if let cornerRadius = config.cornerRadius {
            vfcConfig.cornerRadius = cornerRadius
        }
        if let title = config.title {
            if let hidden = title.hidden {
                vfcConfig.title.isHidden = hidden
            }
            if let textcolor = title.textColor, let color = textcolor.uicolor() {
                vfcConfig.title.textColor = color
            }
            if let numberOfLines = title.numberOfLines {
                vfcConfig.title.numberOfLines = numberOfLines
            }
            if let fontSize = title.fontSize {
                let iOSFontInfo = title.iOSFontInfo ?? FontInfo()
                vfcConfig.title.font = iOSFontInfo.getFont(fontSize)
            }
        }
        if let playIcon = config.playIcon {
            if let hidden = playIcon.hidden {
                vfcConfig.playIcon.isHidden = hidden
            }
            if let iconWidth = playIcon.iconWidth {
                vfcConfig.playIcon.iconWidth = iconWidth
            }
        }
        if let position = config.titlePosition {
            switch position {
            case .stacked:
                vfcConfig.titleLayoutConfiguration.titlePosition = .stacked
            default:
                vfcConfig.titleLayoutConfiguration.titlePosition = .nested
            }
        }

        if let titlePadding = config.titlePadding {
            if let top = titlePadding.top {
                vfcConfig.titleLayoutConfiguration.insets.top = top
            }
            if let right = titlePadding.right {
                vfcConfig.titleLayoutConfiguration.insets.right = right
            }
            if let bottom = titlePadding.bottom {
                vfcConfig.titleLayoutConfiguration.insets.bottom = bottom
            }
            if let left = titlePadding.left {
                vfcConfig.titleLayoutConfiguration.insets.left = left
            }
        }

        if let showAdBadge = config.showAdBadge {
            vfcConfig.sponsored.isHidden = !showAdBadge
        }

        if let enableAutoplay = config.enableAutoplay {
            vfcConfig.autoplay.isEnabled = enableAutoplay
        }

        return vfcConfig
    }

    // swiftlint:disable:next cyclomatic_complexity function_body_length
    static func convertToVideoPlayerContentConfiguration(
        _ config: VideoPlayerConfiguration?
    ) -> VideoPlayerContentConfiguration? {
        guard let config = config else {
            return nil
        }

        var vpcConfig = VideoPlayerContentConfiguration()
        if let playerStyle = config.playerStyle {
            switch playerStyle {
            case .full:
                vpcConfig.playerStyle = .fullBleed
            default:
                vpcConfig.playerStyle = .fit
            }
        }
        if let videoCompleteAction = config.videoCompleteAction {
            switch videoCompleteAction {
            case .loop:
                vpcConfig.videoCompleteAction = .loop
            default:
                vpcConfig.videoCompleteAction = .advanceToNext
            }
        }
        if let showShareButton = config.showShareButton {
            vpcConfig.shareButton.isHidden = !showShareButton
        }
        if let ctaButtonStyle = config.ctaButtonStyle {
            if let backgroundColor = ctaButtonStyle.backgroundColor, let color = backgroundColor.uicolor() {
                vpcConfig.ctaButton.contentConfiguration.backgroundColor = color
            }
            if let textcolor = ctaButtonStyle.textColor, let color = textcolor.uicolor() {
                vpcConfig.ctaButton.contentConfiguration.textColor = color
            }
            if let fontSize = ctaButtonStyle.fontSize {
                let iOSFontInfo = ctaButtonStyle.iOSFontInfo ?? FontInfo()
                vpcConfig.ctaButton.contentConfiguration.font = iOSFontInfo.getFont(fontSize)
            }
        }
        if let showPlaybackButton = config.showPlaybackButton {
            vpcConfig.playbackButton.isHidden = !showPlaybackButton
        }
        if let showMuteButton = config.showMuteButton {
            vpcConfig.muteButton.isHidden = !showMuteButton
        }

        if let launchBehavior = FireworkSDKPlugin.videoLaunchBehavior?.behavior() {
            vpcConfig.onFirstLaunch = launchBehavior
        }

        if let showBranding = config.showBranding {
            vpcConfig.videoDetail.fireworkAttribution.isHidden = !showBranding
        }
        if let shareBaseURL = config.shareBaseURL {
            vpcConfig.shareButton.behavior.baseURL = URL(string: shareBaseURL)
        } else if let shareBaseUrl = FireworkSDKPlugin.shareBaseURL {
            vpcConfig.shareButton.behavior.baseURL = URL(string: shareBaseUrl)
        }

        if let ctaDelay = config.ctaDelay,
           let type = ctaDelay.type,
           let value = ctaDelay.value {
            switch type {
            case .constant:
                vpcConfig.ctaButton.behavior.delay = .constant(value)
            case .percentage:
                vpcConfig.ctaButton.behavior.delay = .percentage(Float(value))
            }
        }

        if let ctaHighlightDelay = config.ctaHighlightDelay,
           let type = ctaHighlightDelay.type,
           let value = ctaHighlightDelay.value {
            switch type {
            case .constant:
                vpcConfig.ctaButton.behavior.highlightDelay = .constant(value)
            case .percentage:
                vpcConfig.ctaButton.behavior.highlightDelay = .percentage(Float(value))
            }
        }

        if let ctaWidth = config.ctaWidth {
            vpcConfig.ctaButton.width = ctaWidth.width()
        }

        if let showVideoDetailTitle = config.showVideoDetailTitle {
            vpcConfig.videoDetail.titleConfiguration.isHidden = !showVideoDetailTitle
        }
        if let buttonConfiguration = config.buttonConfiguration {
            if let videoDetailButtonDisplay = buttonConfiguration.videoDetailButton?.getButtonDisplayConfiguration() {
                vpcConfig.videoDetailButtonDisplayConfiguration = videoDetailButtonDisplay
            }

            if let closeButtonDisplay = buttonConfiguration.closeButton?.getButtonDisplayConfiguration() {
                vpcConfig.closeButtonConfiguration.closeDisplay = closeButtonDisplay
            }

            if let muteButtonDisplay = buttonConfiguration.muteButton?.getButtonDisplayConfiguration() {
                vpcConfig.muteButton.muteDisplay = muteButtonDisplay
            }

            if let unmuteButtonDisplay = buttonConfiguration.unmuteButton?.getButtonDisplayConfiguration() {
                vpcConfig.muteButton.unmuteDisplay = unmuteButtonDisplay
            }

            if let playButtonDisplay = buttonConfiguration.playButton?.getButtonDisplayConfiguration() {
                vpcConfig.playbackButton.playDisplay = playButtonDisplay
            }

            if let pauseButtonDisplay = buttonConfiguration.pauseButton?.getButtonDisplayConfiguration() {
                vpcConfig.playbackButton.pauseDisplay = pauseButtonDisplay
            }
        }

        return vpcConfig
    }

    private func convertToVideoFeedContentConfiguration() -> VideoFeedContentConfiguration {
        var videoConfig = VideoFeedContentConfiguration()

        let vfcConfig = Self.convertToVideoFeedItemContentConfiguration(feedViewConfig)
        let vpcConfig = Self.convertToVideoPlayerContentConfiguration(playerViewConfig)
        if let vfcConfig = vfcConfig {
            videoConfig.itemView = vfcConfig
        }
        if let vpcConfig = vpcConfig {
            videoConfig.playerView = vpcConfig
        }

        if let backgroundColor = feedViewConfig?.backgroundColor, let color = backgroundColor.uicolor() {
            videoConfig.backgroundColor = color
        }

        if let adBadge = FireworkSDKPlugin.convertToFWAdBadgeConfiguration(FireworkSDKPlugin.adBadgeConfiguration) {
            videoConfig.adBadge = adBadge
        }

        // set a value different from default value to compatible with FireworkVideo SDK misalign bug
        if videoConfig.itemView.titleLayoutConfiguration.insets == UIEdgeInsets(top: 8, left: 8, bottom: 8, right: 8) {
            videoConfig.itemView.titleLayoutConfiguration.insets = UIEdgeInsets(top: 7, left: 7, bottom: 7, right: 7)
        }

        return videoConfig
    }
}
