import 'dart:async';

import 'models/fw_events.dart';
import 'models/method_names.dart';
import 'models/product.dart';
import 'models/product_info_view_configuration.dart';
import 'models/shopping_cta_result.dart';
import 'utils/fw_global_state.dart';
import 'utils/fw_method_channel_util.dart';
import 'utils/model_utils/fw_events_util.dart';
import 'utils/model_utils/method_names_util.dart';
import 'utils/model_utils/product_info_view_configuration_util.dart';
import 'utils/model_utils/product_util.dart';
import 'utils/model_utils/shopping_cta_result_util.dart';

typedef ShoppingCTACallback = FutureOr<ShoppingCTAResult> Function(
    ShoppingCTAEvent? event);
typedef CustomClickCartIconCallback = FutureOr<void> Function();
typedef UpdateProductDetailsCallback = FutureOr<List<Product>?> Function(
    UpdateProductDetailsEvent? event);
typedef CustomClickLinkButtonCallback = FutureOr<void> Function(
    CustomClickLinkButtonEvent? event);
typedef CustomTapProductCardCallback = FutureOr<void> Function(
    CustomTapProductCardEvent? event);

/// The entry class of video shopping.
class VideoShopping {
  static VideoShopping? _instance;

  /// This callback is triggered when the user clicks the "Add to cart" or "Shop now" button.
  ///
  /// The host app can return an [ShoppingCTAResult] object to tell SDK how to handle the result.
  ShoppingCTACallback? onShoppingCTA;

  /// This callback is triggered when the user clicks the shopping cart icon.
  ///
  /// The host app can customize the click event processing logic of
  /// the shopping cart icon by setting the callback.
  CustomClickCartIconCallback? onCustomClickCartIcon;

  /// This callback is triggered when the video will be shown.
  ///
  /// The host app can return a [Product] list to update the latest product information.
  UpdateProductDetailsCallback? onUpdateProductDetails;

  /// This callback is triggered when the user clicks
  /// the link button next to Add to cart button.
  ///
  /// The host app can customize the click event processing logic of
  /// the link button by setting the callback.
  CustomClickLinkButtonCallback? get onCustomClickLinkButton =>
      _onCustomClickLinkButton;

  set onCustomClickLinkButton(CustomClickLinkButtonCallback? value) {
    _onCustomClickLinkButton = value;
    final nativeMethodName =
        ShoppingNativeMethodName.setCustomClickLinkButtonEnabled.name;
    FWMethodChannelUtil.getShoppingChannel().invokeMethod(
      nativeMethodName,
      value != null,
    );
  }

  CustomClickLinkButtonCallback? _onCustomClickLinkButton;

  CustomTapProductCardCallback? get onCustomTapProductCard =>
      _onCustomTapProductCard;

  set onCustomTapProductCard(CustomTapProductCardCallback? value) {
    _onCustomTapProductCard = value;
    final nativeMethodName =
        ShoppingNativeMethodName.setCustomTapProductCardEnabled.name;
    FWMethodChannelUtil.getShoppingChannel().invokeMethod(
      nativeMethodName,
      value != null,
    );
  }

  CustomTapProductCardCallback? _onCustomTapProductCard;

  /// Defaults to true. You can hide the cart icon by setting this property to false.
  bool get cartIconVisible => _cartIconVisible;
  set cartIconVisible(bool cartIconVisible) {
    _cartIconVisible = cartIconVisible;
    final nativeMethodName = ShoppingNativeMethodName.setCartIconVisible.name;
    if (FWGlobalState.getInstance().sdkInitCalled) {
      FWMethodChannelUtil.getShoppingChannel().invokeMethod(
        nativeMethodName,
        cartIconVisible,
      );
    } else {
      FWGlobalState.getInstance().sdkInitCalledFuture.then((_) {
        FWMethodChannelUtil.getShoppingChannel().invokeMethod(
          nativeMethodName,
          cartIconVisible,
        );
      });
    }
  }

  bool _cartIconVisible = true;

  /// The configuration of product info view.
  ProductInfoViewConfiguration? get productInfoViewConfiguration =>
      _productInfoViewConfiguration;

  set productInfoViewConfiguration(ProductInfoViewConfiguration? value) {
    _productInfoViewConfiguration = value;
    final nativeMethodName =
        ShoppingNativeMethodName.setProductInfoViewConfiguration.name;
    if (FWGlobalState.getInstance().sdkInitCalled) {
      FWMethodChannelUtil.getShoppingChannel().invokeMethod(
        nativeMethodName,
        ProductInfoViewConfigurationUtil.toJson(
          value ?? ProductInfoViewConfiguration(),
        ),
      );
    } else {
      FWGlobalState.getInstance().sdkInitCalledFuture.then((_) {
        FWMethodChannelUtil.getShoppingChannel().invokeMethod(
          nativeMethodName,
          ProductInfoViewConfigurationUtil.toJson(
            value ?? ProductInfoViewConfiguration(),
          ),
        );
      });
    }
  }

  ProductInfoViewConfiguration? _productInfoViewConfiguration;

  static VideoShopping _getInstance() {
    _instance ??= VideoShopping._();
    return _instance!;
  }

  VideoShopping._() {
    FWMethodChannelUtil.getShoppingChannel().setMethodCallHandler((call) async {
      final fwMethod = ShoppingMethodNameUtil.createMethodName(call.method);
      if (fwMethod == null) {
        return null;
      }

      switch (fwMethod) {
        case ShoppingMethodName.ctaButtonClick:
          if (onShoppingCTA != null) {
            final shoppingCTAEvent =
                ShoppingCTAEventUtil.fromJson(call.arguments);
            final result = await onShoppingCTA!.call(shoppingCTAEvent);
            final resultJson = ShoppingCTAResultUtil.toJson(result);
            return resultJson;
          }
          break;
        case ShoppingMethodName.updateProductDetails:
          if (onUpdateProductDetails != null) {
            final updateProductDetailsEvent =
                UpdateProductDetailsEventUtil.fromJson(call.arguments);
            final productList =
                await onUpdateProductDetails!.call(updateProductDetailsEvent) ??
                    [];
            final productJsonList = productList
                .map(
                  (product) => ProductUtil.toJson(product),
                )
                .toList();
            return productJsonList;
          }
          break;
        case ShoppingMethodName.clickCartIcon:
          if (onCustomClickCartIcon != null) {
            await onCustomClickCartIcon!.call();
          }
          break;
        case ShoppingMethodName.customLinkButtonClick:
          if (onCustomClickLinkButton != null) {
            final customClickLinkButtonEvent =
                CustomClickLinkButtonEventUtil.fromJson(call.arguments);
            await onCustomClickLinkButton!.call(customClickLinkButtonEvent);
          }
          break;
        case ShoppingMethodName.customProductCardTap:
          if (onCustomTapProductCard != null) {
            final customTapProductCardEvent =
                CustomTapProductCardEventUtil.fromJson(call.arguments);
            await onCustomTapProductCard!.call(customTapProductCardEvent);
          }
          break;
      }

      return null;
    });
  }

  factory VideoShopping.getInstance() => _getInstance();

  /// Sets cart item [count].
  ///
  /// The count should be greater than or equal to 0.
  /// We just use count to show or hide red indicator on the cart icon.
  /// If cound > 0, we will show the red indicator on the cart icon.
  /// Otherwise, we will hide the red indicator on the cart icon.
  Future<void> setCartItemCount(int count) async {
    if (!FWGlobalState.getInstance().sdkInitCalled) {
      await FWGlobalState.getInstance().sdkInitCalledFuture;
    }
    final nativeMethodName = ShoppingNativeMethodName.setCartItemCount.name;
    FWMethodChannelUtil.getShoppingChannel().invokeMethod(
      nativeMethodName,
      count,
    );
  }
}
